Package: epiregulon.extra
Title: Companion package to epiregulon with additional plotting,
        differential and graph functions
Version: 1.6.0
Authors@R: 
  c(person(given = "Xiaosai", family = "Yao", role = c("aut", "cre"), email = "xiaosai.yao@gmail.com", comment = c(ORCID = "0000-0001-9729-0726")),
    person(given = "Tomasz", family = "Włodarczyk", role = c("aut"), email = "tomwlo@gmail.com", comment = c(ORCID = "0000-0003-1554-9699")),
    person(given = "Timothy", family = "Keyes", role=c("aut"), email="keyes.timothy@gmail.com"),
    person(given = "Shang-Yang", family = "Chen", role = "aut", email = "sychen9584@gmail.com"))
Description: Gene regulatory networks model the underlying gene regulation hierarchies that drive gene expression and observed phenotypes. Epiregulon infers TF activity in single cells by constructing a gene regulatory network (regulons). This is achieved through integration of scATAC-seq and scRNA-seq data and incorporation of public bulk TF ChIP-seq data. Links between regulatory elements and their target genes are established by computing correlations between chromatin accessibility and gene expressions.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: scran, ComplexHeatmap, Matrix, SummarizedExperiment,
        checkmate, circlize, clusterProfiler, ggplot2, ggraph, igraph,
        patchwork, reshape2, scales, scater
Depends: R (>= 4.4), SingleCellExperiment
Suggests: epiregulon, knitr, rmarkdown, parallel, BiocStyle, testthat
        (>= 3.0.0), msigdb, GSEABase, dorothea, scMultiome, S4Vectors,
        scuttle, vdiffr, ggrastr, ggrepel
VignetteBuilder: knitr
URL: https://github.com/xiaosaiyao/epiregulon.extra/
biocViews: GeneRegulation, Network, GeneExpression, Transcription,
        ChipOnChip, DifferentialExpression, GeneTarget, Normalization,
        GraphAndNetwork
Config/testthat/edition: 3
BugReports: https://github.com/xiaosaiyao/epiregulon.extra/issues
LazyData: true
git_url: https://git.bioconductor.org/packages/epiregulon.extra
git_branch: RELEASE_3_22
git_last_commit: d60eef8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:55:45 UTC; biocbuild
Author: Xiaosai Yao [aut, cre] (ORCID: <https://orcid.org/0000-0001-9729-0726>),
  Tomasz Włodarczyk [aut] (ORCID:
    <https://orcid.org/0000-0003-1554-9699>),
  Timothy Keyes [aut],
  Shang-Yang Chen [aut]
Maintainer: Xiaosai Yao <xiaosai.yao@gmail.com>
