# Generated by roxygen2: do not edit by hand

export(addCentrality)
export(buildDiffGraph)
export(buildGraph)
export(calculateJaccardSimilarity)
export(enrichPlot)
export(findDifferentialActivity)
export(findPartners)
export(getSigGenes)
export(normalizeCentrality)
export(permuteGraph)
export(plotActivityDim)
export(plotActivityViolin)
export(plotBubble)
export(plotDiffNetwork)
export(plotEpiregulonNetwork)
export(plotGseaNetwork)
export(plotHeatmapActivity)
export(plotHeatmapRegulon)
export(rankTfs)
export(regulonEnrich)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,colData)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggraph,create_layout)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_label)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(ggraph,theme_graph)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,each_edge)
importFrom(igraph,edge_attr)
importFrom(igraph,edge_attr_names)
importFrom(igraph,ego)
importFrom(igraph,ends)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,incident_edges)
importFrom(igraph,intersection)
importFrom(igraph,layout_with_sugiyama)
importFrom(igraph,list.vertex.attributes)
importFrom(igraph,rewire)
importFrom(igraph,shortest_paths)
importFrom(igraph,similarity)
importFrom(igraph,strength)
importFrom(igraph,subcomponent)
importFrom(igraph,subgraph)
importFrom(igraph,vcount)
importFrom(igraph,vertex_attr)
