# Description

Companion package to `epiregulon` with additional plotting, differential and graph functions

# Installation

```
if(!require(BiocManager)) install.packages("BiocManager")
BiocManager::install("epiregulon.extra")
```
