#' regulon created using `epiregulon` package from reprogram-seq data
#' @name regulon
#' @docType data
#' @usage data(regulon)
#' @format a DFrame.
#' @return a DFrame.
#' @keywords datasets
#' @examples
#' data(regulon)
"regulon"
