globalVariables(c("head", "name", "type", "logP.adj", "Description",
                  "GeneRatio","Odds.Ratio","activity","relative_activity","summary.logFC",
                  "summary.diff","log.FDR","groups"), package = "epiregulon.extra",
    add = TRUE)
